package com.das.modules.auth.service.impl;

import cn.dev33.satoken.secure.BCrypt;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.das.common.captcha.CaptchaUtils;
import com.das.common.config.SaTokenProperties;
import com.das.common.config.SessionUtil;
import com.das.common.exceptions.ServiceException;
import com.das.common.utils.AdminRedisTemplate;
import com.das.modules.auth.domain.dto.ChangePasswordDto;
import com.das.modules.auth.domain.dto.LoginDto;
import com.das.modules.auth.domain.dto.RefreshTokenDto;
import com.das.modules.auth.domain.vo.LoginUserDetailsVo;
import com.das.modules.auth.domain.vo.LoginVo;
import com.das.modules.auth.domain.vo.SysUserVo;
import com.das.modules.auth.entity.SysUser;
import com.das.modules.auth.mapper.SysUserMapper;
import com.das.modules.auth.service.LoginService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.Duration;
import java.util.Date;

/**
 * @author chenhaojie
 * @Description
 */
@Service
public class LoginServiceImpl implements LoginService {

    @Resource
    private SaTokenProperties saTokenProperties;
    @Resource
    private SysUserMapper sysUserMapper;

    @Autowired
    private AdminRedisTemplate adminRedisTemplate;

    @Override
    public LoginUserDetailsVo login(LoginDto loginDto, HttpServletRequest request, HttpServletResponse response){
        String name = loginDto.getUsername();
        String password = loginDto.getPassword();
        String code = loginDto.getCode();
        String key = loginDto.getKey();

        LoginUserDetailsVo loginInfo = new LoginUserDetailsVo();
        LambdaQueryWrapper<SysUser> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(SysUser::getAccount, name);
        SysUser sysUser = sysUserMapper.selectOne(wrapper);
        if (sysUser == null) {
            loginInfo.setMsg("无账号信息");
            return loginInfo;
        }
        if (!CaptchaUtils.checkVerificationCode(key, code, adminRedisTemplate)) {
            loginInfo.setMsg("验证码不正确");
            return loginInfo;
        }
        if (!BCrypt.checkpw(password, sysUser.getPassword())) {
            loginInfo.setMsg("账号密码错误");
            return loginInfo;
        }
        // 更新用户登录时间
        sysUser.setLastLogin(new Date());
        sysUserMapper.updateById(sysUser);
        StpUtil.login(sysUser.getAccount());// 执行登录，这里username为用户唯一标识
        String refreshTokenUuid = IdUtil.fastSimpleUUID();
        String token = StpUtil.getTokenValue().replace("-", "");
        String refreshToken = "refresh:" + refreshTokenUuid;

        SysUserVo sysUserVo = new SysUserVo();
        sysUserVo.setId(sysUser.getId());
        sysUserVo.setAccount(sysUser.getAccount());
        sysUserVo.setUserName(sysUser.getUserName());
        sysUserVo.setHeadImage(sysUser.getHeadImage());
        sysUserVo.setPhone(sysUser.getPhone());
        sysUserVo.setEmail(sysUser.getEmail());
        sysUserVo.setOrgId(sysUser.getOrgId());

        //查询用户权限列表
        sysUserVo.setAuthorities(sysUserMapper.queryAuthoritiesByUserId(sysUser.getId()));

        loginInfo.setSysUser(sysUserVo); // 存储用户信息到会话
        loginInfo.setToken(token);
        loginInfo.setRefreshToken(refreshTokenUuid);

        adminRedisTemplate.setEx(refreshToken, loginInfo, Duration.ofSeconds(saTokenProperties.getRefreshExpireTime()));

        //将登录用户信息存入 Session
        StpUtil.getTokenSession().set(SessionUtil.SESSION_USER_KEY, sysUserVo);

        return loginInfo;
    }


    @Override
    public void logout(HttpServletRequest request) {
        String token = request.getHeader("token");
        adminRedisTemplate.del(token);
        StpUtil.logoutByTokenValue(token);
    }

    @Override
    public LoginVo refreshToken(RefreshTokenDto refreshToken) {
        LoginUserDetailsVo loginUserDetailsVo = adminRedisTemplate.get(refreshToken.getRefreshToken());
        LoginVo loginInfo = new LoginVo();
        if (loginUserDetailsVo == null) {
            loginInfo.setMsg("token过期，请重新登录");
            return loginInfo;
        }
        StpUtil.login(loginUserDetailsVo.getSysUser().getAccount());// 执行登录，这里username为用户唯一标识
        String newToken = StpUtil.getTokenValue().replace("-", "");
        String newRefreshTokenUuid = IdUtil.fastSimpleUUID();
        String newRefreshToken = "refresh:" + newRefreshTokenUuid;
        loginInfo.setAccessToken(newToken);

        loginUserDetailsVo.setToken(newToken);
        loginUserDetailsVo.setRefreshToken(newRefreshTokenUuid);
        String oldRefreshToken = loginUserDetailsVo.getRefreshToken();

        adminRedisTemplate.setEx(newRefreshToken, loginUserDetailsVo, Duration.ofSeconds(saTokenProperties.getRefreshExpireTime()));
        // 删除原有刷新token
        adminRedisTemplate.del("refresh:" + oldRefreshToken);

        return new LoginVo(newToken, newRefreshToken, saTokenProperties.getRefreshExpireTime());

    }

    @Override
    public int changePassword(ChangePasswordDto changePasswordDto) {
        if (changePasswordDto.getId() == null) {
            throw new ServiceException("非法调用，参数缺失");
        }
        SysUser sysUser = sysUserMapper.selectById(changePasswordDto.getId());
        if (sysUser == null) {
            throw new ServiceException("用户不存在");
        }
        if (!BCrypt.checkpw(changePasswordDto.getOldPassword(), sysUser.getPassword())) {
            throw new ServiceException("原密码不正确");
        }
        String newPassword;
        if (StringUtils.hasText(changePasswordDto.getNewPassword())) {
            newPassword = BCrypt.hashpw(changePasswordDto.getNewPassword(), BCrypt.gensalt());
            sysUser.setPassword(newPassword);
        }
        sysUser.setUpdatedBy(sysUser.getAccount());
        sysUser.setUpdatedTime(new Date());
        return sysUserMapper.updateById(sysUser);
    }

}
